<?php
use Drupal\user\Entity\User;
use Drupal\bibcite_entity\Entity\Reference;
use Drupal\bibcite_import_orcid\Fetch;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\bibcite_import_orcid\Import;

/**
 * Prepare Import Publications
 *
 */
function bibcite_prep_import_orcid_batch($work, $user, &$context) {
  $fetch = new Fetch();
  $uid = $user->id();
  $orcid = trim($user->get('field_orcid')->value);
  $summary = $work['work-summary'][0];
  $orcid_type = $summary["type"];
  $title = substr($summary['title']['title']['value'], 0, 250);
  $publication_date = $summary['publication-date']['year']['value'];
  $work_put_code = $summary['put-code'];
  $type = $fetch->bibcite_entity_types($orcid_type);
  $work_data = $fetch->getOrcidWorkData($work_put_code, $orcid);
  $description = $work_data['short-description'];
  $contributors = $fetch->get_orcid_authors($work_data, $orcid);

  // Search for the reference in bibcite
  $query_refs = \Drupal::entityQuery('bibcite_reference')
  ->condition('title', $title)
  ->condition('type', $type)
  ->execute();

  // If the publication is new, add an option for each contributor
  if (empty($query_refs)){
    foreach ($contributors as $contributor) {
        $options[] = $contributor;
    }
  } else {
    $ref_authors = $fetch->get_reference_authors($query_refs, $uid, $contributors);
    [$options, $attributes] = $ref_authors;

    // Get the user contributor ids
    $user_authors_ids = array_column($user->field_author->getValue(),'target_id','target_id');
    }

    // Prepare output
    $reference_data = [
        'type' => $type,
        'title' => $title,
        'secundary_title' => preg_replace(array('/\s{2,}/', '/[\t\n]/'), ' ', $work_data['journal-title']['value']),
        'bibcite_year' => $publication_date,
        'work_code' => $work_put_code,
        'bibcite_abst_e' => $description,
        'contribs' => $options,
        'user_authors' => $user_authors_ids,
        'contribs_atts' => $attributes,
    ];

    $work_ext_references = $work_data['external-ids']['external-id'];

    // add more references if available
    foreach ($work_ext_references as $ext_ref) {
        switch ($ext_ref['external-id-type']) {
            case 'doi':
                $reference_data['bibcite_url'] = $ext_ref['external-id-url'];
                $reference_data['bibcite_doi'] = $ext_ref['external-id-value'];
                break;
            case 'issn':
                $reference_data['bibcite_issn'] = $ext_ref['external-id-value'];
                break;
            case 'isbn':
                $reference_data['bibcite_isbn'] = $ext_ref['external-id-value'];
                break;
            default:
                $reference_data['bibcite_other_number'] = $ext_ref['external-id-value'];
                break;
        }
    }

    // Return info
    $context['results']['work'][] = json_encode($reference_data);

  }

/**
 * Complete a batch process.
 *
 * @param bool $success
 *   A boolean indicating whether the batch has completed successfully.
 * @param array $results
 *   The value set in $context['results'] by callback_batch_operation().
 * @param array|bool $operations
 *   If $success is FALSE, contains the operations that remained unprocessed.
 */
function bibcite_prep_import_orcid_batch_finished($success, $results, $operations){
    if ($success) {
        $id = time();
        $_SESSION['works_data_' . $id ] = $results['work'];
        return new RedirectResponse('/orcid-import/import/' . $id);
    }else{
        $message = t('An error occured while preparing the import from ORCID. Check the logs for more information. ') . $operations . t(' items to be processed.');
        $messenger = \Drupal::messenger();
        $messenger->addMessage($message);
    }
}

/**
 * Import Publications
 *
 */
function bibcite_import_orcid_batch($work, $uid, &$context) {
    $work_details = json_decode($work['pub'], true);
    $selected_author = $work['author'];
    $pub_authors = [];
    $import = new Import;
    if (isset($selected_author)) {
        foreach ($work_details["contribs"] as $key => $value) {
            $existing_user = $work_details["contribs_atts"] ? array_keys($work_details["contribs_atts"])[0] : NULL;
            $id = $key === $existing_user ? $key : $import->author_to_id($value);
            $pub_authors[] = intval($id);
            if ($key == $selected_author) {
                $user = User::load($uid);
                assert($user instanceof User);
                $authors = $user->field_author->getValue();
                $current_author_ids = isset($authors) ? array_column($authors, 'target_id') : [];
                if (!in_array($id, $current_author_ids)) {
                    $user->field_author[] = ['target_id' => $id];
                    $user->save();
                }
            }
        }
    }

    $title = $work_details['title'];
    $type = $work_details['type'];

    // Check if the reference is already on bibcite
    $query_reference = \Drupal::entityQuery('bibcite_reference')
    ->condition('title', $title)
    ->condition('type', $type)
    ->execute();

    if (empty($query_reference)) {
        // create a new reference
        $bibcite_reference = Reference::create($work_details);
        $context['results']['create'][] = $title;
        // Check the setting for default status
        $default_status = Drupal::config('bibcite_import_orcid.settings')->get('orcid_unpub_default');
        $default_status ? $bibcite_reference->set('status', 0) : NULL;
    } else {
        // load existing and update info if it exists
        $bibcite_reference = Reference::load(reset($query_reference));
        isset($work_details["bibtex"]["bibcite_pages"]) ? $bibcite_reference->bibcite_pages->setValue($work_details["bibtex"]["bibcite_pages"]) : NULL;
        isset($work_details["secundary_title"]) ? $bibcite_reference->bibcite_secondary_title->setValue($work_details["secundary_title"]) : NULL;
        isset($work_details["bibtex"]["bibcite_booktitle"]) ? $bibcite_reference->bibcite_secondary_title->setValue($work_details["bibtex"]["bibcite_booktitle"]) : NULL;
        isset($work_details["bibtex"]["bibcite_volume"]) ? $bibcite_reference->bibcite_volume->setValue($work_details["bibtex"]["bibcite_volume"]) : NULL;
        isset($work_details["bibtex"]["bibcite_section"]) ? $bibcite_reference->bibcite_section->setValue($work_details["bibtex"]["bibcite_section"]) : NULL;
        isset($work_details["bibtex"]["bibcite_publisher"]) ? $bibcite_reference->bibcite_publisher->setValue($work_details["bibtex"]["bibcite_publisher"]) : NULL;
        isset($work_details["bibcite_url"]) ? $bibcite_reference->bibcite_url->setValue($work_details["bibcite_url"]) : NULL;
        isset($work_details["bibcite_other_number"]) ? $bibcite_reference->bibcite_other_number->setValue($work_details["bibcite_other_number"]) : NULL;
        isset($work_details["bibcite_issn"]) ? $bibcite_reference->bibcite_issn->setValue($work_details["bibcite_issn"]) : NULL;
        isset($work_details["bibcite_isbn"]) ? $bibcite_reference->bibcite_isbn->setValue($work_details["bibcite_isbn"]) : NULL;
        isset($work_details["bibcite_doi"]) ? $bibcite_reference->bibcite_doi->setValue($work_details["bibcite_doi"]) : NULL;
        $context['results']['update'][] = $title;
    }
    $pub_authors ? $bibcite_reference->set('author', $pub_authors) : NULL;
    $bibcite_reference->save();
}

/**
 * Complete a batch process.
 *
 * @param bool $success
 *   A boolean indicating whether the batch has completed successfully.
 * @param array $results
 *   The value set in $context['results'] by callback_batch_operation().
 * @param array|bool $operations
 *   If $success is FALSE, contains the operations that remained unprocessed.
 */
function bibcite_import_orcid_batch_finished($success, $results, $operations) {

    // The 'success' parameter means no fatal PHP errors were detected. All
    // other error management should be handled using 'results'.
    if ($success) {
        // Cleanup duplicate authors
        $user_id = \Drupal::currentUser()->id();
        $user = User::load($user_id);
        assert($user instanceof User);
        $authors = $user->field_author->getValue();
        $authors = array_column($authors, 'target_id');
        $authors = array_unique($authors);
        $user->field_author = $authors;
        $user->save();

        $id = time();
        $_SESSION['results_orcid_' . $id ] = $results;
        return new RedirectResponse('/orcid-import/success/' . $id);
    }else{
        $message = t('An error occured while importing.');
        }
    $messenger = \Drupal::messenger();
    $messenger->addMessage($message);
}
