<?php

namespace Drupal\bibcite_import_orcid\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Drupal\bibcite_import_orcid\Fetch;

/**
 * Provides a ORCID Prepare form.
 */
class PrepareImport extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'prepare_import';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['submit'] = array('#type' => 'submit', '#value' => t('Import from ORCID'));
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $uid = \Drupal::routeMatch()->getParameter('user')->id();
    $user = User::load($uid);
    $orcid = trim($user->get('field_orcid')->value);
    $fetch = new Fetch();
    $works = $fetch->getAllWorksFromOrcid($orcid);

    $_SESSION['uid'] = $uid;
    // Prepare form batch

    $batch = array(
      'title' => t('Fetching publications from ORCID, please wait...'),
      'operations' => [],
      'finished' => 'bibcite_prep_import_orcid_batch_finished',
      'init_message' => t('Preparing the form'),
      'progress_message' => t('@current of @total. Estimated duration: @estimate.'),
      'error_message' => t('There was an error on the import process.'),
      'file' => \Drupal::service('extension.list.module')->getPath('bibcite_import_orcid') . '/bibcite_import_orcid.batch.inc',
    );
    foreach ($works as $work) {
        $batch['operations'][] = ['bibcite_prep_import_orcid_batch', [$work, $user]];
    };
    batch_set($batch);
  }
}
