<?php

namespace Drupal\bibcite_import_orcid;

use Drupal;
use Drupal\user\Entity\User;
use Drupal\bibcite_entity\Entity\Contributor;

/**
 * Import ORCID data.
 */
class Import {

  /**
   * Given a author name, look for an author.
   * @return int
   */
  public function author_to_id($name){
    if(!is_int($name)){
      // Parse the name
      if(str_contains($name,',')){
        $names = explode(',', $name);
        $cfname = ucfirst(end($names));
        $clname = reset($names);
      } else {
        $names = explode(" ", $name);
        $cfname = reset($names);
        $clname = ucfirst(end($names));
      }
      $label = $cfname . ' ' . $clname;

      // Check if the contributor exists
      $query = Drupal::entityQuery('bibcite_contributor')
      ->accessCheck(FALSE)
      ->condition('first_name', $cfname)
      ->condition('last_name', $clname)
      ->execute();

      if(!empty($query)){
        // Check if there's already an attached author with the same name
        $user_id = \Drupal::currentUser()->id();
        $user = User::load($user_id);
        $author_ids = $user->field_author->getValue();
        $author_labels = [];
        foreach ($author_ids as $author_id) {
          $auth_id = $author_id["target_id"];
          $storage = \Drupal::entityTypeManager()->getStorage("bibcite_contributor");
          $author = $storage->load($auth_id);
          if($author){
            $author_labels[$author_id["target_id"]] = $author->label();
          }
        }
        if (in_array(trim($label),$author_labels)) {
          foreach ($author_labels as $key => $author_label) {
            if($author_label == trim($label)){
              return $key;
            }
          }
          return reset($author_id);
        } else {
          return reset($query);
        }
      }
      //Create a contributor
      $c = Contributor::create([
        'type' => 'entity_contributor',
        'last_name' => $clname,
        'first_name' => $cfname,
      ]);
      $c->save();
      $id = $c->id();
      return $id;
    }
  }
}
